
import ProSW.*;
public class Azienda {
	
	private /*@ spec_public non_null @*/  final String nome;
	private /*@ spec_public @*/ String sede;
	private /*@ spec_public non_null @*/ final String pIVA;
	private /*@ spec_public @*/ int capSociale;
	//@ invariant capSociale>0;
	
	/*@  requires n!=null && iva!=null;
		 assignable nome,sede,pIVA,capSociale;
		 ensures (pIVA == iva) && (sede == s) && (capSociale == cap) && (nome == n);
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public Azienda(String n, String s, String iva, int cap) throws EccezionePrecondizioni {
		if (n==null) throw new EccezionePrecondizioni("Il nome non puo' essere null");
		nome=n;
		setSede(s);
		if (iva==null) throw new EccezionePrecondizioni("La p. iva non puo' essere null");
		pIVA=iva;
		setCapSociale(cap);
	}
	public /*@  pure  @*/ String  getNome() { return nome; }
	public /*@  pure  @*/ String  getSede() { return sede; }
	public /*@  pure  @*/ String  getpIVA() { return pIVA; }
	public /*@  pure  @*/ int     getCapSociale()  { return capSociale; }
	
	/*@  requires s!=null;
		 assignable sede;
		 ensures sede == s;
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public void setSede(String s) throws EccezionePrecondizioni { 
		if (s==null) throw new EccezionePrecondizioni("La sede non puo' essere null");
		sede=s; 
	}
	/*@  requires cap >0;
		 assignable capSociale;
		 ensures capSociale == cap;
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public void setCapSociale(int cap) throws EccezionePrecondizioni {
		if (cap<=0) throw new EccezionePrecondizioni("Il capitale sociale deve essere positivo");
		capSociale=cap;	
	}
	
	/*@  requires  capSociale > 10000000;
		 ensures \result == true;
		 also
		 requires  capSociale <= 10000000;
		 ensures \result == false;	 
	 @*/
	public /* pure */boolean isMedioGrande() {
		return capSociale > 10000000;
	}
} //:~
