
import ProSW.*;

public abstract class Collezione {	

	protected /*@ spec_public @*/ final int codiceCollezione;
	protected /*@ spec_public @*/ String temaDominante;
	protected /*@ spec_public @*/ final int anno;
	
	//* invariant anno>0;
	
	/*@  requires a>0 && s !=null;
		 assignable codiceCollezione,temaDominante,anno,appartiene;
		 ensures (codiceCollezione == cod) && (temaDominante == tema) && (anno == a) && (appartiene == s);
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public Collezione(int cod, String tema, int a, Stilista s) throws EccezionePrecondizioni { 
     	codiceCollezione = cod;
		setTemaDominante(tema);
		if (a <= 0) throw new EccezionePrecondizioni("anno non valido");
		anno=a;
		if (s==null) throw new EccezionePrecondizioni("Lo stilista non puo' essere null");
		appartiene = s;
	}
	public /*@  pure  @*/ int getCodiceCollezione() { return codiceCollezione; }
	
	public /*@  pure  @*/ String getTemaDominante() { return temaDominante; }
	
	public /*@  pure  @*/ int getAnno() { return anno; }
	
	/*@  requires t!=null;
		 assignable temaDominante;
		 ensures temaDominante == t;
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public void setTemaDominante(String t) throws EccezionePrecondizioni { 
		if (t==null) throw new EccezionePrecondizioni("Il tema non puo' essere null");
		temaDominante=t; 
	}
	
	// Gestione associazione `appartiene'
	private /*@ spec_public  @*/ Stilista appartiene = null;
	//* invariant appartiene!=null
	
	public /*@  pure  @*/ Stilista getAppartiene() { 
		return appartiene; 
		// Si noti come posso evitare di generare EccezioneMolteplicita
		// anche se il vincolo di molt. e' 1..1.
		// Questo perche' ho deciso che il link e' immutabile e noto alla nascita
		// ed e' il costruttore che evita che il cliente passi uno stilista `null'.
		//nella specifica JML abbiamo usato l'invariant
	}
	
	public /*@  pure  @*/ String toString() {
		return "Collezione " + codiceCollezione + "(" + this.getClass().getName() + "). Tema: " + temaDominante + ", " + anno + ", stilista " + appartiene;
	}
	
	
} //:~