

import ProSW.*;
import java.util.*;

public class Sfilata {
	private /*@ spec_public @*/ String luogo;
	private /*@ spec_public @*/ Data dataInizio;
	private /*@ spec_public @*/ Data dataFine;
	
	/*@	assignable luogo,dataInizio,dataFine;
	 	ensures luogo == l && dataInizio == di && dataFine == df;
	    signals_only EccezionePrecondizioni;
	 @*/
	public Sfilata(String l, Data di, Data df) throws EccezionePrecondizioni {
		setLuogo(l);
		setDataInizio(di);
		setDataFine(df);
	}
		
	public /*@  pure  @*/ Data getDataInizio() { return dataInizio;}
	 
	public /*@  pure  @*/ Data getDataFine()   { return dataFine;}
	
	public /*@  pure  @*/ String getLuogo()    { return luogo;}
	
	/*@ 
	     requires d!=null;
	     assignable dataInizio;
		 ensures dataInizio == d;		 
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public void setDataInizio(Data d) throws EccezionePrecondizioni {
		if (d==null) throw new EccezionePrecondizioni("La data non puo' essere null");
 		dataInizio = d;  
	}
	
	/*@ 
	     requires d!=null;
	     assignable dataFine;
		 ensures dataFine == d;		 
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public void setDataFine(Data d) throws EccezionePrecondizioni {
		if (d==null) throw new EccezionePrecondizioni("La data non puo' essere null");
 		dataFine = d; 
	}
	
	/*@ 
	     requires l!=null;
	     assignable luogo;
		 ensures luogo == l;		 
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public void setLuogo(String l) throws EccezionePrecondizioni { 
		if (l==null) throw new EccezionePrecondizioni("Il luogo non puo' essere null");
		luogo=l; 
	}

	// Gestione associazione Presentata (1..*)
	private /*@ spec_public @*/ Set linkPresentata = new HashSet();


	/*@  requires collezione!=null;
	     ensures  linkPresentata.contains(collezione);
		 signals_only EccezionePrecondizioni;	 
	 @*/
	public void inserisciLinkPresentata(Collezione collezione) 
		  throws EccezionePrecondizioni {
		if (collezione == null) throw new EccezionePrecondizioni("La collezione non puo' essere null");
	  	linkPresentata.add( collezione );
	}
	
	/*@  requires linkPresentata.size() >= 1;
		 ensures \result == linkPresentata;	
		 assignable \nothing;	 
		 signals_only EccezioneMolteplicita;	 
	 @*/
	public /* pure */ Set getLinkPresentata() throws EccezioneMolteplicita {
		if (linkPresentata.size() < 1)
			throw new EccezioneMolteplicita("I vincoli di molteplicita' dell'associazione 'Presentata' sono violati");
		return (Set)((HashSet)linkPresentata).clone(); 
	}
	
	public String toString() {
		String result = "Sfilata in " + luogo + "(" + dataInizio + " - " + dataFine + ")";
		try {
			Iterator it = getLinkPresentata().iterator();
			while (it.hasNext()) {
				// '\n' e' il carattere di ritorno a capo
				Collezione c = (Collezione) it.next();
				result += "\n - " + c;
			}
		} catch(EccezioneMolteplicita em) {
			result += "\n" + em + "\nIgnoro";
		}
		return result;
	}
	
} //:~